package Pod::L10N;

use vars qw($VERSION);
$VERSION = '1.03';

=pod

=head1 NAME

Pod::L10N - Pod with L10N

=head1 AUTHOR

SHIRAKATA Kentaro E<lt>argrath@cpan.orgE<gt>

=head1 LICENSE

This program is distributed under the Artistic License.
